
#include <windows.h>  
#include <commctrl.h>
#include "trackbar.h"  


#if defined (WIN32)
	#define IS_WIN32 TRUE
#else
	#define IS_WIN32 FALSE
#endif

#define IS_NT      IS_WIN32 && (BOOL)(GetVersion() < 0x80000000)
#define IS_WIN32S  IS_WIN32 && (BOOL)(!(IS_NT) && (LOBYTE(LOWORD(GetVersion()))<4))
#define IS_WIN95   (BOOL)(!(IS_NT) && !(IS_WIN32S)) && IS_WIN32

HINSTANCE hInst;   // current instance

LPCTSTR lpszAppName = "MyApp";
LPCTSTR lpszTitle   = "Track Bar"; 


BOOL RegisterWin95( CONST WNDCLASS* lpwc );


int APIENTRY WinMain( HINSTANCE hInstance, HINSTANCE hPrevInstance,
                      LPTSTR lpCmdLine, int nCmdShow)
{
   MSG      msg;
   HWND     hWnd; 
   WNDCLASS wc;

   wc.style         = CS_HREDRAW | CS_VREDRAW;
   wc.lpfnWndProc   = (WNDPROC)WndProc;       
   wc.cbClsExtra    = 0;                      
   wc.cbWndExtra    = 0;                      
   wc.hInstance     = hInstance;              
   wc.hIcon         = LoadIcon (hInstance, lpszAppName); 
   wc.hCursor       = LoadCursor(NULL, IDC_ARROW);
   wc.hbrBackground = (HBRUSH)(COLOR_3DFACE+1);
   wc.lpszMenuName  = lpszAppName;              
   wc.lpszClassName = lpszAppName;              

   if ( IS_WIN95 )
   {
      if ( !RegisterWin95( &wc ) )
         return( FALSE );
   }
   else if ( !RegisterClass( &wc ) )
      return( FALSE );

   hInst = hInstance; 

   hWnd = CreateWindow( lpszAppName, 
                        lpszTitle,    
                        WS_OVERLAPPEDWINDOW, 
                        CW_USEDEFAULT, 0, 
                        CW_USEDEFAULT, 0,  
                        NULL,              
                        NULL,              
                        hInstance,         
                        NULL               
                      );

   if ( !hWnd ) 
      return( FALSE );

   ShowWindow( hWnd, nCmdShow ); 
   UpdateWindow( hWnd );         

   while( GetMessage( &msg, NULL, 0, 0) )   
   {
      TranslateMessage( &msg ); 
      DispatchMessage( &msg );  
   }

   return( msg.wParam ); 
}


BOOL RegisterWin95( CONST WNDCLASS* lpwc )
{
   WNDCLASSEX wcex;

   wcex.style         = lpwc->style;
   wcex.lpfnWndProc   = lpwc->lpfnWndProc;
   wcex.cbClsExtra    = lpwc->cbClsExtra;
   wcex.cbWndExtra    = lpwc->cbWndExtra;
   wcex.hInstance     = lpwc->hInstance;
   wcex.hIcon         = lpwc->hIcon;
   wcex.hCursor       = lpwc->hCursor;
   wcex.hbrBackground = lpwc->hbrBackground;
   wcex.lpszMenuName  = lpwc->lpszMenuName;
   wcex.lpszClassName = lpwc->lpszClassName;

   // Added elements for Windows 95.
   //...............................
   wcex.cbSize = sizeof(WNDCLASSEX);
   wcex.hIconSm = LoadImage(wcex.hInstance, lpwc->lpszClassName, 
                            IMAGE_ICON, 16, 16,
                            LR_DEFAULTCOLOR );
			
   return RegisterClassEx( &wcex );
}




LRESULT CALLBACK WndProc( HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam )
{
static HWND hTrack = NULL;

   switch( uMsg )
   {
      case WM_CREATE :
              InitCommonControls();

              // Create the edit control.
              //.........................
              hTrack = CreateWindowEx( 0, TRACKBAR_CLASS, "",
                                       WS_CHILD | WS_VISIBLE |
                                       TBS_ENABLESELRANGE | TBS_AUTOTICKS,
                                       40, 20, 200, 30, hWnd, 
                                       (HMENU)IDC_TRACKBAR, hInst, NULL );

              if ( hTrack )
                 SendMessage( hTrack, TBM_SETRANGE, TRUE, MAKELPARAM(0,10) );

              break;


      case WM_COMMAND :
              switch( LOWORD( wParam ) )
              {
                 case IDM_PROPERTIES :
                        DialogBox( hInst, "OptionsDlg", hWnd, (DLGPROC)Options );
                        break;

                 case IDM_RESET :
                        SendMessage( hTrack, TBM_SETRANGE, TRUE, MAKELPARAM(0,10) );
                        SendMessage( hTrack, TBM_CLEARSEL, TRUE, 0 );
                        SendMessage( hTrack, TBM_CLEARTICS, TRUE, 0 );
                        break;

                 case IDM_ABOUT :
                        DialogBox( hInst, "AboutBox", hWnd, (DLGPROC)About );
                        break;

                 case IDM_EXIT :
                        DestroyWindow( hWnd );
                        break;
              }
              break;
      
      case WM_DESTROY :
              PostQuitMessage(0);
              break;

      default :
            return( DefWindowProc( hWnd, uMsg, wParam, lParam ) );
   }

   return( 0L );
}


LRESULT CALLBACK Options( HWND hDlg,           
                          UINT message,        
                          WPARAM wParam,       
                          LPARAM lParam)
{
   switch (message) 
   {
       case WM_INITDIALOG: 
               {
                  HWND   hTrack;
                  DWORD  dwLineSize;
                  DWORD  dwNumTics;
                  DWORD  dwPageSize;
                  DWORD  dwPos;
                  DWORD  dwThSize;
                  DWORD  dwMaxRange;
                  DWORD  dwMinRange;
                  DWORD  dwSelStart;
                  DWORD  dwSelEnd;

                  // Retrieve the trackbar handle.
                  //...............................
                  hTrack = GetDlgItem( GetParent( hDlg ), IDC_TRACKBAR );

                  // Retrieve the information from the trackbar.
                  //............................................
                  dwLineSize = SendMessage( hTrack, TBM_GETLINESIZE, 0, 0 );
                  dwNumTics  = SendMessage( hTrack, TBM_GETNUMTICS, 0, 0 );
                  dwPageSize = SendMessage( hTrack, TBM_GETPAGESIZE, 0, 0 );
                  dwPos      = SendMessage( hTrack, TBM_GETPOS, 0, 0 );
                  dwMinRange = SendMessage( hTrack, TBM_GETRANGEMIN, 0, 0 );
                  dwMaxRange = SendMessage( hTrack, TBM_GETRANGEMAX, 0, 0 );
                  dwSelStart = SendMessage( hTrack, TBM_GETSELSTART, 0, 0 );
                  dwSelEnd   = SendMessage( hTrack, TBM_GETSELEND, 0, 0 );
                  dwThSize   = SendMessage( hTrack, TBM_GETTHUMBLENGTH, 0, 0 );

                  // Initialize the dialog controls with the values.
                  //................................................
                  SetDlgItemInt( hDlg, IDC_LINESIZE, dwLineSize, FALSE );
                  SetDlgItemInt( hDlg, IDC_TICKFREQ, 
                                 (dwMaxRange-dwMinRange)/(dwNumTics-1), FALSE );
                  SetDlgItemInt( hDlg, IDC_PAGESIZE, dwPageSize, FALSE );
                  SetDlgItemInt( hDlg, IDC_POS, dwPos, FALSE );
                  SetDlgItemInt( hDlg, IDC_THUMBSIZE, dwThSize, FALSE );
                  SetDlgItemInt( hDlg, IDC_RANGE1, dwMinRange, FALSE );
                  SetDlgItemInt( hDlg, IDC_RANGE2, dwMaxRange, FALSE );
                  SetDlgItemInt( hDlg, IDC_SEL1, dwSelStart, FALSE );
                  SetDlgItemInt( hDlg, IDC_SEL2, dwSelEnd, FALSE );
               }
               return (TRUE);

       case WM_COMMAND:                              
               if ( LOWORD(wParam) == IDOK )
               {
                  HWND   hTrack;
                  DWORD  dwLineSize;
                  DWORD  dwNumTics;
                  DWORD  dwPageSize;
                  DWORD  dwPos;
                  DWORD  dwThSize;
                  DWORD  dwMaxRange;
                  DWORD  dwMinRange;
                  DWORD  dwSelStart;
                  DWORD  dwSelEnd;

                  // Retrieve the trackbar handle.
                  //...............................
                  hTrack = GetDlgItem( GetParent( hDlg ), IDC_TRACKBAR );

                  // Retrieve the new settings from the dialog.
                  //...........................................
                  dwLineSize = GetDlgItemInt( hDlg, IDC_LINESIZE, NULL, FALSE );
                  dwNumTics  = GetDlgItemInt( hDlg, IDC_TICKFREQ, NULL, FALSE );
                  dwPageSize = GetDlgItemInt( hDlg, IDC_PAGESIZE, NULL, FALSE );
                  dwPos      = GetDlgItemInt( hDlg, IDC_POS, NULL, FALSE );
                  dwThSize   = GetDlgItemInt( hDlg, IDC_THUMBSIZE, NULL, FALSE );
                  dwMinRange = GetDlgItemInt( hDlg, IDC_RANGE1, NULL, FALSE );
                  dwMaxRange = GetDlgItemInt( hDlg, IDC_RANGE2, NULL, FALSE );
                  dwSelStart = GetDlgItemInt( hDlg, IDC_SEL1, NULL, FALSE );
                  dwSelEnd   = GetDlgItemInt( hDlg, IDC_SEL2, NULL, FALSE );

                  // Set the new attributes of the trackbar control.
                  //................................................
                  SendMessage( hTrack, TBM_SETLINESIZE, 0, dwLineSize );
                  SendMessage( hTrack, TBM_SETTICFREQ, dwNumTics, 0 );
                  SendMessage( hTrack, TBM_SETPAGESIZE, 0, dwPageSize );
                  SendMessage( hTrack, TBM_SETPOS, TRUE, dwPos );
                  SendMessage( hTrack, TBM_SETRANGEMIN, TRUE, dwMinRange );
                  SendMessage( hTrack, TBM_SETRANGEMAX, TRUE, dwMaxRange );
                  SendMessage( hTrack, TBM_SETSELSTART, TRUE, dwSelStart );
                  SendMessage( hTrack, TBM_SETSELEND, TRUE, dwSelEnd );
                  SendMessage( hTrack, TBM_SETTHUMBLENGTH, dwThSize, 0 );

                  EndDialog( hDlg, IDOK );        
               }
               else if ( LOWORD(wParam) == IDCANCEL )    
                  EndDialog( hDlg, IDCANCEL );        
               break;
   }

   return (FALSE); 
}


LRESULT CALLBACK About( HWND hDlg,           
                        UINT message,        
                        WPARAM wParam,       
                        LPARAM lParam)
{
   switch (message) 
   {
       case WM_INITDIALOG: 
               return (TRUE);

       case WM_COMMAND:                              
               if (   LOWORD(wParam) == IDOK         
                   || LOWORD(wParam) == IDCANCEL)    
               {
                       EndDialog(hDlg, TRUE);        
                       return (TRUE);
               }
               break;
   }

   return (FALSE); 
}
